@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------- 
- * iperf, Copyright (c) 2014, 2016, The Regents of the University of
+ * iperf, Copyright (c) 2014-2018, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -91,23 +91,24 @@ extern    "C"
  * usage
  * ------------------------------------------------------------------- */
 
-const char usage_shortstr[] = "Usage: iperf [-s|-c host] [options]\n"
-                           "Try `iperf --help' for more information.\n";
+const char usage_shortstr[] = "Usage: iperf3 [-s|-c host] [options]\n"
+                           "Try `iperf3 --help' for more information.\n";
 
-const char usage_longstr[] = "Usage: iperf [-s|-c host] [options]\n"
-                           "       iperf [-h|--help] [-v|--version]\n\n"
+const char usage_longstr[] = "Usage: iperf3 [-s|-c host] [options]\n"
+                           "       iperf3 [-h|--help] [-v|--version]\n\n"
                            "Server or Client:\n"
                            "  -p, --port      #         server port to listen on/connect to\n"
-                           "  -f, --format    [kmgKMG]  format to report: Kbits, Mbits, KBytes, MBytes\n"
-                           "  -i, --interval  #         seconds between periodic bandwidth reports\n"
+                           "  -f, --format   [kmgtKMGT] format to report: Kbits, Mbits, Gbits, Tbits\n"
+                           "  -i, --interval  #         seconds between periodic throughput reports\n"
                            "  -F, --file name           xmit/recv the specified file\n"
 #if defined(HAVE_CPU_AFFINITY)
                            "  -A, --affinity n/n,m      set CPU affinity\n"
 #endif /* HAVE_CPU_AFFINITY */
-                           "  -B, --bind      <host>    bind to a specific interface\n"
+                           "  -B, --bind      <host>    bind to the interface associated with the address <host>\n"
                            "  -V, --verbose             more detailed output\n"
                            "  -J, --json                output in JSON format\n"
                            "  --logfile f               send output to a log file\n"
+                           "  --forceflush              force flushing output at every interval\n"
                            "  -d, --debug               emit debugging output\n"
                            "  -v, --version             show version information and quit\n"
                            "  -h, --help                show this message and quit\n"
@@ -116,6 +117,12 @@ const char usage_longstr[] = "Usage: iperf [-s|-c host] [options]\n"
                            "  -D, --daemon              run the server as a daemon\n"
                            "  -I, --pidfile file        write PID file\n"
                            "  -1, --one-off             handle one client connection then exit\n"
+#if defined(HAVE_SSL)
+                           "  --rsa-private-key-path    path to the RSA private key used to decrypt\n"
+			   "                            authentication credentials\n"
+                           "  --authorized-users-path   path to the configuration file containing user\n"
+                           "                            credentials\n"
+#endif //HAVE_SSL
                            "Client specific:\n"
                            "  -c, --client    <host>    run in client mode, connecting to <host>\n"
 #if defined(HAVE_SCTP)
@@ -124,14 +131,20 @@ const char usage_longstr[] = "Usage: iperf [-s|-c host] [options]\n"
                            "  --nstreams      #         number of SCTP streams\n"
 #endif /* HAVE_SCTP */
                            "  -u, --udp                 use UDP rather than TCP\n"
-                           "  -b, --bandwidth #[KMG][/#] target bandwidth in bits/sec (0 for unlimited)\n"
+                           "  --connect-timeout #       timeout for control connection setup (ms)\n"
+                           "  -b, --bitrate #[KMG][/#]  target bitrate in bits/sec (0 for unlimited)\n"
                            "                            (default %d Mbit/sec for UDP, unlimited for TCP)\n"
                            "                            (optional slash and packet count for burst mode)\n"
+			   "  --pacing-timer #[KMG]     set the timing for pacing, in microseconds (default 1000)\n"
+#if defined(HAVE_SO_MAX_PACING_RATE)
+                           "  --fq-rate #[KMG]          enable fair-queuing based socket pacing in\n"
+			   "                            bits/sec (Linux only)\n"
+#endif
                            "  -t, --time      #         time in seconds to transmit for (default %d secs)\n"
                            "  -n, --bytes     #[KMG]    number of bytes to transmit (instead of -t)\n"
                            "  -k, --blockcount #[KMG]   number of blocks (packets) to transmit (instead of -t or -n)\n"
-                           "  -l, --len       #[KMG]    length of buffer to read or write\n"
-			   "                            (default %d KB for TCP, %d KB for UDP)\n"
+                           "  -l, --length    #[KMG]    length of buffer to read or write\n"
+			   "                            (default %d KB for TCP, dynamic or %d for UDP)\n"
                            "  --cport         <port>    bind to a specific client port (TCP and UDP, default: ephemeral port)\n"
                            "  -P, --parallel  #         number of parallel client streams to run\n"
                            "  -R, --reverse             run in reverse mode (server sends, client receives)\n"
@@ -143,20 +156,30 @@ const char usage_longstr[] = "Usage: iperf [-s|-c host] [options]\n"
                            "  -N, --no-delay            set TCP/SCTP no delay, disabling Nagle's Algorithm\n"
                            "  -4, --version4            only use IPv4\n"
                            "  -6, --version6            only use IPv6\n"
-                           "  -S, --tos N               set the IP 'type of service'\n"
+                           "  -S, --tos N               set the IP type of service, 0-255.\n"
+                           "                            The usual prefixes for octal and hex can be used,\n"
+                           "                            i.e. 52, 064 and 0x34 all specify the same value.\n"
+
+                           "  --dscp N or --dscp val    set the IP dscp value, either 0-63 or symbolic.\n"
+                           "                            Numeric values can be specified in decimal,\n"
+                           "                            octal and hex (see --tos above).\n"
 #if defined(HAVE_FLOWLABEL)
                            "  -L, --flowlabel N         set the IPv6 flow label (only supported on Linux)\n"
 #endif /* HAVE_FLOWLABEL */
                            "  -Z, --zerocopy            use a 'zero copy' method of sending data\n"
                            "  -O, --omit N              omit the first n seconds\n"
                            "  -T, --title str           prefix every output line with this string\n"
+                           "  --extra-data str          data string to include in client and server JSON\n"
                            "  --get-server-output       get results from server\n"
                            "  --udp-counters-64bit      use 64-bit counters in UDP test packets\n"
-#if defined(HAVE_SO_MAX_PACING_RATE)
-                           "  --no-fq-socket-pacing     disable fair-queuing based socket pacing\n"
-			   "                            (Linux only)\n"
-#endif
-
+                           "  --repeating-payload       use repeating pattern in payload, instead of\n"
+                           "                            randomized payload (like in iperf2)\n"
+#if defined(HAVE_SSL)
+                           "  --username                username for authentication\n"
+                           "  --rsa-public-key-path     path to the RSA public key used to encrypt\n"
+                           "                            authentication credentials\n"
+#endif //HAVESSL
+    
 #ifdef NOT_YET_SUPPORTED /* still working on these */
 #endif
 
@@ -228,13 +251,13 @@ const char wait_server_threads[] =
 "Waiting for server threads to complete. Interrupt again to force quit.\n";
 
 const char test_start_time[] =
-"Starting Test: protocol: %s, %d streams, %d byte blocks, omitting %d seconds, %d second test\n";
+"Starting Test: protocol: %s, %d streams, %d byte blocks, omitting %d seconds, %d second test, tos %d\n";
 
 const char test_start_bytes[] =
-"Starting Test: protocol: %s, %d streams, %d byte blocks, omitting %d seconds, %llu bytes to send\n";
+"Starting Test: protocol: %s, %d streams, %d byte blocks, omitting %d seconds, %llu bytes to send, tos %d\n";
 
 const char test_start_blocks[] =
-"Starting Test: protocol: %s, %d streams, %d byte blocks, omitting %d seconds, %d blocks to send\n";
+"Starting Test: protocol: %s, %d streams, %d byte blocks, omitting %d seconds, %d blocks to send, tos %d\n";
 
 
 /* -------------------------------------------------------------------
@@ -247,6 +270,12 @@ const char report_time[] =
 const char report_connecting[] =
 "Connecting to host %s, port %d\n";
 
+const char report_authetication_successed[] =
+"Authentication successed for user '%s' ts %ld\n";
+
+const char report_authetication_failed[] =
+"Authentication failed for user '%s' ts %ld\n";
+
 const char report_reverse[] =
 "Reverse mode, remote host %s is sending\n";
 
@@ -281,19 +310,19 @@ const char report_read_length_times[] =
 "[%3d] %5d bytes read %5d times (%.3g%%)\n";
 
 const char report_bw_header[] =
-"[ ID] Interval           Transfer     Bandwidth\n";
+"[ ID] Interval           Transfer     Bitrate\n";
 
 const char report_bw_retrans_header[] =
-"[ ID] Interval           Transfer     Bandwidth       Retr\n";
+"[ ID] Interval           Transfer     Bitrate         Retr\n";
 
 const char report_bw_retrans_cwnd_header[] =
-"[ ID] Interval           Transfer     Bandwidth       Retr  Cwnd\n";
+"[ ID] Interval           Transfer     Bitrate         Retr  Cwnd\n";
 
 const char report_bw_udp_header[] =
-"[ ID] Interval           Transfer     Bandwidth       Jitter    Lost/Total Datagrams\n";
+"[ ID] Interval           Transfer     Bitrate         Jitter    Lost/Total Datagrams\n";
 
 const char report_bw_udp_sender_header[] =
-"[ ID] Interval           Transfer     Bandwidth       Total Datagrams\n";
+"[ ID] Interval           Transfer     Bitrate         Total Datagrams\n";
 
 const char report_bw_format[] =
 "[%3d] %6.2f-%-6.2f sec  %ss  %ss/sec                  %s\n";
@@ -364,6 +393,9 @@ const char report_local[] = "local";
 const char report_remote[] = "remote";
 const char report_sender[] = "sender";
 const char report_receiver[] = "receiver";
+const char report_sender_not_available_format[] = "[%3d] (sender statistics not available)\n";
+const char report_sender_not_available_summary_format[] = "[%3s] (sender statistics not available)\n";
+const char report_receiver_not_available_format[] = "[%3d] (receiver statistics not available)\n";
 
 #if defined(linux)
 const char report_tcpInfo[] =
